/*****************************************************************************
Author:Peter J. Vis
First Written:8 Dec 2004
Last Updated: 12 Dec 2006
Title: Displaying text on a 128x64 LCD

Microcontroller:ATmega32
Crystal:16Mhz
Platform:Development System

URL:http://petervis.co.cc/C/128x64/128x64.html

LIMITATIONS:
No portion of this work can be used in commercial
applications without prior written permission.This program can be published in
magazines and articles with previous consent and as long as credit is given
to the author.

COPYRIGHT:	
All the material including function names are copyright.

LIABILITY:
Obviously none.

SUPPORT:
Feel free to contact me if you are developing a product and wish to employ me. 

POSTCARD:
Send me a postcard / email if you found my code useful. 

PURPOSE:
This program and its associated files 128x64.c and 128x64.h were created to
show how the LCD could be used to display a bitmapped graphic.

CIRCUIT:
My Modular Development System

*******************************************************************************/


#include <avr/pgmspace.h>
#include <avr/io.h>


// crystal
#define F_CPU 16000000UL

// Control Port
#define	cPort	 	PORTB
#define	cPort_ddr	DDRB
#define	cPort_pins	PINB

// Data Port
#define	dPort		PORTA
#define	dPort_ddr	DDRA
#define	dPort_pins	PINA


//Control Port Bits
						// Function ------------- LCD ------ PortB
#define	lcdrs	0 		// RS ALSO CALLED D/I --- pin4 ----- Pin0 
#define	lcdrw	1 		// R/W ------------------ pin5 ----- Pin1 
#define	lcde	2       // e -------------------- pin6 ----- Pin2 
#define	lcdcs1	5       // CS1 ------------------ pin15 ---- Pin5 
#define	lcdcs2	4       // CS2 ------------------ pin16 ---- Pin4 
#define	lcdrst	3       // RST ------------------ pin17 ---- Pin3 


// LCD Commands
#define START_LINE		0b11000000
#define SET_ADDRESS		0b01000000	// column address to 0
#define SET_PAGE		0b10111000	// page address to 0
#define DISP_ON			0b00111111
#define DISP_OFF		0b00111110


// global variables
unsigned char column = 0;
unsigned char page = 0;


// fuction prototypes
void cSelect(uint8_t col);				// Select the controller chip depending on column number

// command pair
void CommandMode (void);				// lcd data port to accept commands
void WriteCommand(uint8_t cmd);			// send a command to the controllers

// data pair
void DataMode (void);					// lcd data port to accept pixel data
void LocknLoad (void);					// load the data buffer and latch for next byte

// initialise lcd
void lcdInit (void);					// initialise lcd
void clearl (void);						// clear lcd						

void setNewPage (unsigned char PageData);

// set cursor position
void setXY(uint8_t x, uint8_t y);


// font and text display						
void lcd_writech(uint8_t ch); 			// main character driver
void printl(const char* str);			// lcd user printing function


